@@ -9,9 +9,11 @@ import matplotlib.pyplot as plt
 import sys
 import os
 import textwrap
+import json
 
 from quakesranalysis import scanhandler
 from quakesranalysis import gaussianmixture
+from quakesranalysis import mixturefit
 from quakesranalysis import allan
 
 def printUsage():
@@ -29,9 +31,59 @@ def printUsage():
         \t-wndnoise N\tPlots noise in a sliding window of N samples
         \t-offsettime\tPlot offset change (mean of all samples) over time
         \t-allan\tPlot allan deviations for all sampled points along main axis
-        \t-decompose\tDecompose gaussian mixture for all channels
+        \t-mixfit\tDecompose into Gaussian and Cauchy distributions for all channels
+        \t-mixfitdebug\tDump all fitting steps (might require MPLBACKEND=tkagg instead of qt?agg)
+        \t-decompose\tDecompose gaussian mixture for all channels (old fitter)
+        \t-metrics\tCollect core metrics in metrics.json for this run
     """).format(sys.argv[0]))
 
+def metrics_collect_core(fileprefix, scan, metrics = {}):
+    #   Allan deviation -> converges or diverges? How well is it a 1/r?
+
+    metrics['core'] = {}
+
+    # Spans
+
+    meanI, stdI, meanQ, stdQ = scan.get_signal_mean_iq()
+    meanIZ, stdIZ, meanQZ, stdQZ = scan.get_zero_mean_iq()
+    if meanIZ is not None:
+        meanIDiff, stdIDiff, meanQDiff, stdQDiff = scan.get_diff_mean_iq()
+
+    metrics['core']['vpp_signalI'] = np.max(meanI) - np.min(meanI)
+    metrics['core']['vpp_signalQ'] = np.max(meanQ) - np.min(meanQ)
+    metrics['core']['mean_signalI'] = np.mean(meanI)
+    metrics['core']['mean_signalQ'] = np.mean(meanQ)
+    metrics['core']['maxnoise_signalI'] = np.max(stdI)
+    metrics['core']['maxnoise_signalQ'] = np.max(stdQ)
+    metrics['core']['meannoise_signalI'] = np.mean(stdI)
+    metrics['core']['meannoise_signalQ'] = np.mean(stdQ)
+    if meanIZ is not None:
+        metrics['core']['vpp_signalIZero'] = np.max(meanIZ) - np.min(meanIZ)
+        metrics['core']['vpp_signalQZero'] = np.max(meanQZ) - np.min(meanQZ)
+        metrics['core']['vpp_signalIDiff'] = np.max(meanIDiff) - np.min(meanIDiff)
+        metrics['core']['vpp_signalQDiff'] = np.max(meanQDiff) - np.min(meanQDiff)
+        metrics['core']['mean_signalIZero'] = np.mean(meanIZ)
+        metrics['core']['mean_signalQZero'] = np.mean(meanQZ)
+        metrics['core']['mean_signalIDiff'] = np.mean(meanIDiff)
+        metrics['core']['mean_signalQDiff'] = np.mean(meanQDiff)
+        metrics['core']['maxnoise_signalIZero'] = np.max(stdIZ)
+        metrics['core']['maxnoise_signalQZero'] = np.max(stdQZ)
+        metrics['core']['maxnoise_signalIDiff'] = np.max(stdIDiff)
+        metrics['core']['maxnoise_signalQDiff'] = np.max(stdQDiff)
+        metrics['core']['meannoise_signalIZero'] = np.mean(stdIZ)
+        metrics['core']['meannoise_signalQZero'] = np.mean(stdQZ)
+        metrics['core']['meannoise_signalIDiff'] = np.mean(stdIDiff)
+        metrics['core']['meannoise_signalQDiff'] = np.mean(stdQDiff)
+
+    return metrics
+
+def metrics_write(fileprefix, scan, metrics):
+    # Write metrics file with all collected metrics ...
+    with open(f"{fileprefix}_metrics.json", "w") as outfile:
+        outfile.write(json.dumps(metrics))
+
+    print(f"[METRICS] Written {fileprefix}_metrics.json")
+
 def plot_allan(fileprefix, scan):
     print(f"[ALLAN] Starting for {fileprefix}")
 
@@ -53,7 +105,7 @@ def plot_allan(fileprefix, scan):
         ax[i][0].grid()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_allanall.svg")
+    ##plt.savefig(f"{fileprefix}_allanall.svg")
     plt.savefig(f"{fileprefix}_allanall.png")
     plt.close(fig)
     print(f"[ALLAN] Written {fileprefix}_allanall")
@@ -68,19 +120,88 @@ def plot_allan(fileprefix, scan):
     ax.grid()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_allan.svg")
+    ##plt.savefig(f"{fileprefix}_allan.svg")
     plt.savefig(f"{fileprefix}_allan.png")
     plt.close(fig)
     print(f"[ALLAN] Written {fileprefix}_allan")
 
+    fig, ax = plt.subplots()
+    ax.set_title(f"Worst Allan deviation (from all {scan.get_main_axis_symbol()})")
+    ax.set_ylabel("Allan deviation")
+    ax.set_xlabel("Time (samples)")
+    ax.set_yscale('log')
+    ax.set_xscale('log')
+    for channel in worstres:
+        ax.plot(worstres[channel]['taus'], worstres[channel]['ad'], label = f"{channel}")
+    ax.legend()
+    ax.grid()
 
-def plot_decompose(fileprefix, scan, debugplots = False):
+    plt.tight_layout()
+    #plt.savefig(f"{fileprefix}_allan_log.svg")
+    plt.savefig(f"{fileprefix}_allan_log.png")
+    plt.close(fig)
+    print(f"[ALLAN] Written {fileprefix}_allan_log")
+
+
+def plot_decompose_mixturefit(fileprefix, scan, metrics = {}, debugplots = False):
+    print(f"[MIXFIT] Starting for {fileprefix}")
+
+    mf = mixturefit.MixtureFitter()
+
+    # Decompose for each channel in the scan ...
+    scanx = scan.get_main_axis_data()
+    scanxsymbol = scan.get_main_axis_symbol()
+
+    meanI, _, meanQ, _ = scan.get_signal_mean_iq()
+    amp, _ = scan.get_signal_ampphase()
+
+    meanIZero, _, meanQZero, _ = scan.get_zero_mean_iq()
+    ampZero, _ = scan.get_zero_ampphase()
+    meanIDiff, _, meanQDiff, _ = scan.get_diff_mean_iq()
+    ampDiff, _ = scan.get_diff_ampphase()
+
+    metrics['decompose'] = {}
+
+    resmixture = {
+        'I' : mf.fitMixture(scanx, meanI, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_I", printprogress = True),
+        'Q' : mf.fitMixture(scanx, meanQ, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_Q", printprogress = True),
+        'A' : mf.fitMixture(scanx, amp, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_A", printprogress = True)
+    }
+
+    if meanIZero is not None:
+        resmixture = resmixture | {
+            'IZero' : mf.fitMixture(scanx, meanIZero, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_IZero", printprogress = True),
+            'QZero' : mf.fitMixture(scanx, meanQZero, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_QZero", printprogress = True),
+            'AZero' : mf.fitMixture(scanx, ampZero, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_AZero", printprogress = True),
+
+            'IDiff' : mf.fitMixture(scanx, meanIDiff, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_IDiff", printprogress = True),
+            'QDiff' : mf.fitMixture(scanx, meanQDiff, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_QDiff", printprogress = True),
+            'ADiff' : mf.fitMixture(scanx, ampDiff, debugplots = debugplots, debugplotsshow = False, debugplotsprefix = f"{fileprefix}_mixfit_ADiff", printprogress = True)
+        }
+
+    chanlist = [ 'I', 'Q', 'A' ]
+    if meanIZero is not None:
+        chanlist = chanlist + [ 'IZero' , 'QZero' , 'AZero', 'IDiff', 'QDiff', 'ADiff' ]
+
+    for chan in chanlist:
+        fig, ax = mf.mixtures2barplot(resmixture[chan], yunitlabel = scanxsymbol, chanlabel = chan)
+        plt.savefig(f"{fileprefix}_mixfit_{chan}.png")
+        plt.close()
+        print(f"[MIXFIT] Written {fileprefix}_mixfit_{chan}")
+
+        metrics['decompose'][chan] = mf.mixtures2dictlist(resmixture[chan])
+
+    print("[MIXFIT] Done")
+    return metrics
+
+
+def plot_decompose(fileprefix, scan, metrics = {}, debugplots = False):
     print(f"[DECOMPOSE] Starting for {fileprefix}")
 
     res = gaussianmixture.decompose_gaussian_mixtures(scan, progressPrint = True, debugplots = debugplots, debugplotPrefix=f"{fileprefix}_fitdebug_")
     fig, ax = gaussianmixture.decompose_gaussian_mixtures_plotfit(scan, res)
 
-    plt.savefig(f"{fileprefix}_gaussianmixture_fit.svg")
+    #plt.savefig(f"{fileprefix}_gaussianmixture_fit.svg")
     plt.savefig(f"{fileprefix}_gaussianmixture_fit.png")
     print(f"[DECOMPOSE] Written {fileprefix}_gaussianmixture_fit")
     plt.close(fig)
@@ -88,12 +209,24 @@ def plot_decompose(fileprefix, scan, debugplots = False):
     # Show fit results
     print("[DECOMPOSE]")
     print("[DECOMPOSE] \tCenter\tFWHM\tOffset\tAmplitude")
+
+    metrics['decompose'] = {}
+
     for channel in res:
+        metrics['decompose'][channel] = []
+
         res[channel] = sorted(res[channel], key=lambda x : x['mu'])
         print(f"[DECOMPOSE] Channel {channel}")
         for params in res[channel]:
             fwhm = 2.0 * np.sqrt(2.0 * np.log(2)) * params['sigma']
             print(f"[DECOMPOSE] \t{params['mu']}\t{fwhm}\t{params['off']}\t{params['amp']}")
+            metrics['decompose'][channel].append({
+                'type' : params['type'],
+                'mu' : params['mu'],
+                'fwhm' : fwhm,
+                'off' : params['off'],
+                'amp' : params['amp']
+            })
     print("[DECOMPOSE]")
 
     # Generate component plots if debugplots is enabled
@@ -110,11 +243,19 @@ def plot_decompose(fileprefix, scan, debugplots = False):
             outfile.write("#Center\tFWHM\tOffset\tAmplitude\n")
             for params in res[channel]:
                 fwhm = 2.0 * np.sqrt(2.0 * np.log(2)) * params['sigma']
-                outfile.write(f"{params['mu']}\t{fwhm}\t{params['off']}\t{params['amp']}\n")
+                typestring = "Unknown"
+                if params['type'] == gaussianmixture.FUNTYPE_DIFFGAUSSIAN:
+                    typestring = "Diff. Gaussian"
+                elif params['type'] == gaussianmixture.FUNTYPE_GAUSSIAN:
+                    typestring = "Gaussian"
+
+                outfile.write(f"{typestring}\t{params['mu']}\t{fwhm}\t{params['off']}\t{params['amp']}\n")
         print(f"[DECOMPOSE] Written {fileprefix}_gaussianmixture_peaks_{channel}.dat")
 
     print("[DECOMPOSE] Finished")
 
+    return metrics
+
 
 def plot_offsettime(fileprefix, scan):
     # Calculate offset drift over time
@@ -239,7 +380,7 @@ def plot_offsettime(fileprefix, scan):
         ax[2][2].grid()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_offsettime.svg")
+    #plt.savefig(f"{fileprefix}_offsettime.svg")
     plt.savefig(f"{fileprefix}_offsettime.png")
     plt.close(fig)
     print(f"[OFFSETTIME] Written {fileprefix}_offsettime")
@@ -373,7 +514,7 @@ def plot_wndnoise(fileprefix, scan, job):
         ax[2][1].legend()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_wndnoise_{wndSize}.svg")
+    #plt.savefig(f"{fileprefix}_wndnoise_{wndSize}.svg")
     plt.savefig(f"{fileprefix}_wndnoise_{wndSize}.png")
     plt.close(fig)
     print(f"[WNDNOISE] Written {fileprefix}_wndnoise_{wndSize}")
@@ -401,7 +542,7 @@ def plot_ampphase(fileprefix, scan):
     ax[1].grid()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_signal_ap.svg")
+    #plt.savefig(f"{fileprefix}_signal_ap.svg")
     plt.savefig(f"{fileprefix}_signal_ap.png")
     print(f"[AMPHASE] Written {fileprefix}_signal")
     plt.close(fig)
@@ -423,7 +564,7 @@ def plot_ampphase(fileprefix, scan):
         ax[1].grid()
 
         plt.tight_layout()
-        plt.savefig(f"{fileprefix}_zero_ap.svg")
+        #plt.savefig(f"{fileprefix}_zero_ap.svg")
         plt.savefig(f"{fileprefix}_zero_ap.png")
         print(f"[AMPHASE] Written {fileprefix}_zero_ap")
         plt.close(fig)
@@ -444,7 +585,7 @@ def plot_ampphase(fileprefix, scan):
         ax[1].grid()
 
         plt.tight_layout()
-        plt.savefig(f"{fileprefix}_diff_ap.svg")
+        #plt.savefig(f"{fileprefix}_diff_ap.svg")
         plt.savefig(f"{fileprefix}_diff_ap.png")
         print(f"[AMPHASE] Written {fileprefix}_diff_ap")
         plt.close(fig)
@@ -474,7 +615,7 @@ def plot_iqmean(fileprefix, scan):
     ax[1].grid()
 
     plt.tight_layout()
-    plt.savefig(f"{fileprefix}_signal.svg")
+    #plt.savefig(f"{fileprefix}_signal.svg")
     plt.savefig(f"{fileprefix}_signal.png")
     print(f"[IQMEAN] Written {fileprefix}_signal")
     plt.close(fig)
@@ -498,7 +639,7 @@ def plot_iqmean(fileprefix, scan):
         ax[1].grid()
 
         plt.tight_layout()
-        plt.savefig(f"{fileprefix}_zero.svg")
+        #plt.savefig(f"{fileprefix}_zero.svg")
         plt.savefig(f"{fileprefix}_zero.png")
         print(f"[IQMEAN] Written {fileprefix}_zero")
         plt.close(fig)
@@ -521,7 +662,7 @@ def plot_iqmean(fileprefix, scan):
         ax[1].grid()
 
         plt.tight_layout()
-        plt.savefig(f"{fileprefix}_diff.svg")
+        #plt.savefig(f"{fileprefix}_diff.svg")
         plt.savefig(f"{fileprefix}_diff.png")
         print(f"[IQMEAN] Written {fileprefix}_diff")
         plt.close(fig)
@@ -550,8 +691,14 @@ def main():
             jobs.append({ 'task' : 'decompose', 'debug' : False })
         elif sys.argv[i].strip() == "-decomposedebug":
             jobs.append({ 'task' : 'decompose', 'debug' : True })
+        elif sys.argv[i].strip() == "-mixfit":
+            jobs.append({ 'task' : 'mixfit', 'debug' : False })
+        elif sys.argv[i].strip() == "-mixfitdebug":
+            jobs.append({ 'task' : 'mixfit', 'debug' : True })
         elif sys.argv[i].strip() == "-allan":
             jobs.append({ 'task' : 'allan' })
+        elif sys.argv[i].strip() == "-metrics":
+            jobs.append({ 'task' : 'metrics' })
         elif sys.argv[i].strip() == "-wndnoise":
             n = 0
             if i == (len(sys.argv)-1):
@@ -596,6 +743,7 @@ def main():
         # Execute jobs ...
         scans = sc.get_scans()
         for iscan, scan in enumerate(scans):
+            metrics = {}
             if len(scans) > 2:
                 fnprefix = f"{jobfile}"
             else:
@@ -610,9 +758,14 @@ def main():
                 if job['task'] == "offsettime":
                     plot_offsettime(fnprefix, scan)
                 if job['task'] == "decompose":
-                    plot_decompose(fnprefix, scan, debugplots = job['debug'])
+                    metrics = plot_decompose(fnprefix, scan, metrics = metrics, debugplots = job['debug'])
                 if job['task'] == "allan":
                     plot_allan(fnprefix, scan)
+                if job['task'] == "metrics":
+                    metrics = metrics_collect_core(fnprefix, scan, metrics)
+                    metrics_write(fnprefix, scan, metrics)
+                if job['task'] == "mixfit":
+                    metrics = plot_decompose_mixturefit(fnprefix, scan, metrics = metrics, debugplots = job['debug'])
 
 if __name__ == "__main__":
     main()
\ No newline at end of file
